#include "OpenGlPainter.h"

#include "DrawingContext.h"
#include "DrawingSurface.h"
#include "Scene.h"
#include "TriMesh.h"

#ifdef _WIN32
#include <windows.h>
#endif

#include <GL/gl.h>

#include <iostream>

void OpenGlPainter::paint(DrawingContext* context)
{
    auto surface = context->getSurface();
    const auto width = double(surface->getWidth());
    const auto height = double(surface->getHeight());

    std::cout << "Painting into width " << width << " and height " << height << std::endl;

    const auto num_mesh = context->getScene()->getNumMeshes();

    glClearColor(1.0, 1.0, 1.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT);

    //glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

    for (std::size_t idx=0; idx<num_mesh; idx++)
    {
        auto mesh = context->getScene()->getMesh(idx);
        const auto faces =  mesh->getFaceVertices();
        const auto colors = mesh->getFaceVectorAttributes("Color");

        std::size_t counter{0};
        for (const auto& face : faces)
        {
            const auto r = colors[counter][0];
            const auto g = colors[counter][1];
            const auto b = colors[counter][2];

            glColor3f(r, g, b);

            glBegin(GL_TRIANGLES);
            double x0 = 2.0*face[0].getX() / width - 1.0;
            double y0 = 1.0 - 2.0*face[0].getY() / height;

            double x1 = 2.0*face[1].getX() / width - 1.0;
            double y1 = 1.0  - 2.0*face[1].getY() / height;

            double x2 = 2.0*face[2].getX() / width - 1.0;
            double y2 = 1.0  - 2.0*face[2].getY() / height;

            glVertex3f(x0, y0, 0);
            glVertex3f(x1, y1, 0);
            glVertex3f(x2, y2, 0);

            // std::cout << "Verts0| " << x0 << " | " << y0 << " | "<< std::endl;
            // std::cout << "Verts1| " << x1 << " | " << y1 << " | "<< std::endl;
            // std::cout << "Verts2| " << x2 << " | " << y2 << " | "<< std::endl;
            // std::cout << "****************" << std::endl;

            glEnd();

            counter++;
        }
    }

    glFlush();
}
