#include "TriMesh.h"

#include "Node.h"
#include "Edge.h"
#include "TriFace.h"

TriMesh::~TriMesh()
{

}

void TriMesh::populate(VecNodes& nodes, VecEdges& edges, VecFaces& faces)
{
    mNodes = std::move(nodes);
    mEdges = std::move(edges);
    mFaces = std::move(faces);
}

std::vector<std::vector<Point> > TriMesh::getFaceVertices() const
{
    std::vector<std::vector<Point> > verts(mFaces.size());

    for(std::size_t idx=0; idx<mFaces.size(); idx++)
    {
        const auto nodeIds = mFaces[idx]->getNodeIds();
        verts[idx] = {mNodes[nodeIds[0]]->getPoint(), mNodes[nodeIds[1]]->getPoint(), mNodes[nodeIds[2]]->getPoint()};
    }
    return verts;
}

std::vector<std::vector<double> > TriMesh::getFaceVectorAttributes(const std::string& tag)
{
    std::vector<std::vector<double> > attribs(mFaces.size());
    for(std::size_t idx=0; idx<mFaces.size(); idx++)
    {
        attribs[idx] = {mFaces[idx]->getVectorAttribute(tag)};
    }
    return attribs;
}

void TriMesh::addConstantFaceVectorAttribute(const std::string& tag, const std::vector<double>& values)
{
    for (const auto& face : mFaces)
    {
        face->addVectorAttribute(tag, values);
    }
}
