#pragma once

#include "AbstractMesh.h"
#include "Point.h"

#include <vector>
#include <memory>

class Node;
class Edge;
class TriFace;

using NodePtr = std::unique_ptr<Node>;
using EdgePtr = std::unique_ptr<Edge>;
using TriFacePtr = std::unique_ptr<TriFace>;

using VecNodes = std::vector<NodePtr>;
using VecEdges = std::vector<EdgePtr>;
using VecFaces = std::vector<TriFacePtr>;

class TriMesh : public AbstractMesh
{
public:
    TriMesh() = default;

    ~TriMesh();

    void populate(VecNodes& nodes, VecEdges& edges, VecFaces& faces);

    std::vector<std::vector<Point> > getFaceVertices() const;

    void addConstantFaceVectorAttribute(const std::string& tag, const std::vector<double>& values);

    std::vector<std::vector<double> > getFaceVectorAttributes(const std::string& tag);

private:
    VecNodes mNodes;
    VecEdges mEdges;
    VecFaces mFaces;
};
