#pragma once

#include "AbstractVisualNode.h"
#include "Color.h"

class GeometryNode : public AbstractVisualNode
{
public:
    enum class Type
    {
        Path,
        Rectangle,
        Circle,
        Arc
    };

public:
    GeometryNode(const DiscretePoint& location);
    virtual ~GeometryNode() = default;

    const Color& getFillColor() const;
    const Color& getStrokeColor() const;
    unsigned getStrokeThickness() const;
    virtual Type getType() = 0;

    void setFillColor(const Color& color);
    void setStrokeColor(const Color& color);
    void setStrokeThickness(unsigned thickness);
private:
    Color mFillColor;
    Color mStrokeColor;
    unsigned mStrokeThickness{0};
    Type mType;
};

using GeometryNodePtr = std::unique_ptr<GeometryNode>;
